@@ -96,7 +96,7 @@ void aliens_create_wave_purple(alien* aliens, int num)
 	int i;
 	for(i=0; i < num; i++)
 	{
-		char foo[100];
+		//char foo[100];
 		alien_purple* ap;
 		alien* a = &aliens[i];
 
@@ -130,7 +130,7 @@ void aliens_create_wave_purple(alien* aliens, int num)
 void aliens_draw(game* g)
 {
 	int i;
-	char foo[100];
+	//char foo[100];
 	s32 dx=0, dy=0;
 
 	for(i=0; i < g->num_aliens; i++)
@@ -194,7 +194,7 @@ void aliens_draw(game* g)
 							alien_bullet* b = &g->alien_bullets[i];
 							s32 dx, dy;
 							s32 dist, scale_factor;
-							char foo[100];
+							//char foo[100];
 							
 							b->x = a->x; b->y = a->y;
 							b->distance_travelled = 0;
