@@ -3,21 +3,21 @@ typedef struct
 	s32 x, y;
 	s16 direction;
 	Sprite sprite;
-	u16 distance_total, distance_travelled, speed;
+	u32 distance_total, distance_travelled, speed;
 	u16 active;
 } bullet;
 
-#define BULLET_SPEED 4
+#define BULLET_SPEED FIXED_INT(4)
 
 typedef struct 
 {
 	s32 x, y;
 	u16 direction;
 	Sprite sprite;
-	u16 xspeed;
-	s16 min_x, max_x;
-	u16 yspeed;
-	s16 min_y, max_y;
+	u32 xspeed;
+	s32 min_x, max_x;
+	u32 yspeed;
+	s32 min_y, max_y;
 	u16 num_bullets;
 	bullet* bullets;
 	u16 fire_delay;
