@@ -24,10 +24,12 @@ typedef struct
 	u32 length, cur;
 	u16 volume; // volume value from 0-16
 	s16 panning; // panning value from -8 to 8
-	u32 volcache[2]; // volume cache mixing code
+	u16 volcache[2]; // volume cache mixing code
+	u32 freq;				// playback frequency
 } buffer_sound;
 
 #define SOUNDBUFFER_CHANNELS 6
+
 buffer_sound sounds[SOUNDBUFFER_CHANNELS];
 
 // init_sfx_system - sets the registers for enabling the sound hardware and
@@ -69,8 +71,8 @@ void init_sfx_system(void)
 }
 
 void dump_samples()
-{
-	/*int i;
+{/*
+	int i;
 	char foo[100];
 
 	for(i=0; i < SOUNDBUFFER_CHANNELS; i++)
@@ -78,15 +80,58 @@ void dump_samples()
 		sprintf(foo, "channel: %d\n", i); print(foo);
 		sprintf(foo, "\tlen: %d\n", sounds[i].length); print(foo);
 		sprintf(foo, "\tcur: %d\n", sounds[i].cur); print(foo);
+		sprintf(foo, "\tvol: %d\n", sounds[i].volume); print(foo);
+		sprintf(foo, "\tpan: %d\n", sounds[i].panning); print(foo);
+		sprintf(foo, "\tfre: %d\n", sounds[i].freq); print(foo);
 	}*/
 }
 
-void process_soundbuffer()
+void mix_buffer(u16* vals, const s8* data, u32 vol, u32 mix_len)
 {
+	int i;
+
+	for(i=0; i < mix_len; i++)
+	{
+		vals[i] += ((data[i] * vol) >> 8);
+	}
+}
+
+void downsample_mix_buffer(u16* vals, const s8* data, u32 vol, u32 mix_len, u32 scale_low, u32 scale_high)
+{
+	//unsigned int scale_high = in_freq / out_freq;
+	//unsigned int scale_low = (((in_freq % out_freq) << 16) / out_freq);
+	u32 mix_count=0, mix_pos=0;
+
+	int i;
+
+	for(i=0; i < mix_len; i++)
+	{
+		vals[i] += ((data[mix_pos] * vol) >> 8);
+
+		mix_count += scale_low;
+		if(mix_count > (1 << 16))
+		{
+			mix_count -= (1 << 16);
+			mix_pos++;
+		}
+
+		mix_pos += scale_high;
+	}
+}
+
+void asm_mix_buffer(s16* vals, const s8* data, u32 vol, u32 mix_len) CODE_IN_IWRAM;
+void asm_downsample_mix_buffer(u16* vals, const s8* data, u32 vol, u32 mix_len, u32 scale_low, u32 scale_high) CODE_IN_IWRAM;
+
+void asm_flush_work_buffer(s16* in, s8* out, u32 len) CODE_IN_IWRAM;
+
+u32 process_soundbuffer()
+{
+	//char foo[100];
 	int i, s;
 	int channel;
-	s32 samples_played;
-
+	u32 process_time;
+	u32 samples_played;
+	s16 vals[SOUNDBUFFER_LENGTH];
 	s8* buf[2];
 
 	if(sbuffer)
@@ -100,41 +145,56 @@ void process_soundbuffer()
 
 	//dump_samples();
 
+	REG_TM2D = 0;
+	REG_TM2CNT = TIMER_FREQ_1024 | TIMER_ENABLE;
 	/* pre-calculate volume levels for each channel of each sound */
 	for(s=0; s < SOUNDBUFFER_CHANNELS; s++)
 	{
 		sounds[s].volcache[0] = sounds[s].volume * (8 - sounds[s].panning);
 		sounds[s].volcache[1] = sounds[s].volume * (8 + sounds[s].panning);
 	}
-
+	
 	for(channel=0; channel < 2; channel++)
 	{
-		for(i=0; i < SOUNDBUFFER_LENGTH; i++)
-		{
-			s32 val = 0;
+		memset(vals, 0, SOUNDBUFFER_LENGTH * sizeof(s16));
 		
-			for(s=0; s < SOUNDBUFFER_CHANNELS; s++)
+		for(s=0; s < SOUNDBUFFER_CHANNELS; s++)
+		{
+			if(sounds[s].cur < sounds[s].length)
 			{
-				if((sounds[s].cur + i) < sounds[s].length)
+				u16 mix_len;
+
+				if(sounds[s].freq == FREQUENCY)
 				{
-					val += ((sounds[s].data[sounds[s].cur + i] * sounds[s].volcache[channel]) >> 8);
+					mix_len = sounds[s].length - sounds[s].cur;
+					if(mix_len > SOUNDBUFFER_LENGTH) mix_len = SOUNDBUFFER_LENGTH;
+				
+					asm_mix_buffer(vals, sounds[s].data + sounds[s].cur, sounds[s].volcache[channel], mix_len);
+				} else {
+					u32 scale_low, scale_high;
+					
+					mix_len = (((sounds[s].length - sounds[s].cur) * FREQUENCY) / sounds[s].freq);
+					if(mix_len > SOUNDBUFFER_LENGTH)
+					{
+						mix_len = SOUNDBUFFER_LENGTH;
+					}	
+					scale_high = sounds[s].freq / FREQUENCY;
+					scale_low = (((sounds[s].freq % FREQUENCY) << 16) / FREQUENCY);
+					//sprintf(foo,"sh: %d sl: %d ml: %d\n", scale_high, scale_low, mix_len); print(foo);
+					asm_downsample_mix_buffer(vals, sounds[s].data + sounds[s].cur, sounds[s].volcache[channel], mix_len, scale_low, scale_high);
 				}
 			}
-			//val = val / SOUNDBUFFER_CHANNELS;
-			val = val >> 1;
-			if(val < -128 || val > 128)
-			{
-				//print("clip!\n");
-			}
-			buf[channel][i] = val;
 		}
+		asm_flush_work_buffer(vals, buf[channel], SOUNDBUFFER_LENGTH);
 	}
+	process_time = REG_TM2D;
+	
 	samples_played = 265;
 	for(i=0; i < SOUNDBUFFER_CHANNELS; i++)
 	{
 		if(sounds[i].length)
 		{
-			sounds[i].cur += samples_played;
+			sounds[i].cur += ((samples_played * sounds[i].freq) / FREQUENCY);
 			if(sounds[i].cur > sounds[i].length)
 			{
 				sounds[i].cur = 0; sounds[i].length = 0;
@@ -178,6 +238,8 @@ void process_soundbuffer()
 	// and enable interrupt used to stop the sample playing when it's finished
 	REG_IE |= INT_TM1;
 	REG_IME = 1;
+
+	return(process_time);
 }
 
 void play_sound(const SAMPLE* s)
@@ -193,6 +255,7 @@ void play_sound(const SAMPLE* s)
 			sounds[i].volume = s->volume;
 			sounds[i].panning = s->panning;	
 			sounds[i].length = s->length;
+			sounds[i].freq = s->freq;
 			return;
 		}
 	}
