@@ -26,7 +26,7 @@
 #define DSB_TIMER0            0x0000
 #define DSB_TIMER1            0x4000
 #define DSB_FIFO_RESET        0x8000
-#define FREQUENCY             22050
+#define FREQUENCY             16000
 //#define TIMER_INTERVAL        0xFD07    // (65536-(round((2^24)/FREQUENCY)))
 #define TIMER_INTERVAL        0xFbe8    // (65536-(round((2^24)/FREQUENCY)))
 #define SAMPLES_PER_VBLANK    0x0170    // (FREQUENCY/60)
@@ -37,11 +37,12 @@ typedef struct
 	const s8  *pData;			// pointer to the raw sound data
 	u32 length;						// the size of the sample
 	u16 volume, panning;	// volume and panning (0 - 16 for vol, -8 - 8 for pan)
+	u16 freq;
 } SAMPLE;
 
 // function declarations
 void init_sfx_system(void);
 void play_sound(const SAMPLE *pSample);
-void process_soundbuffer();
+u32 process_soundbuffer();
 
 #endif
